import math
import cv2
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
from matplotlib import cm
from numpy.linalg import inv
from numba import jit

import color_histogram as ch

@jit
def colorSimilarity(input_hsv, hist_model, tsize, step, is_weighted):
    iwidth = input_hsv.shape[1]
    iheight = input_hsv.shape[0]
    twidth = tsize[0]
    theight = tsize[1]
    hist_size = (hist_model.shape[1], hist_model.shape[0])

    similarity_map = np.zeros(((iheight - theight + 1),
                               (iwidth - twidth + 1)), 
                              np.float64)
    swidth = similarity_map.shape[1]
    sheight = similarity_map.shape[0]

    for j in range(0, sheight, step):
        for i in range(0, swidth, step):
            region = input_hsv[j:(j + theight), 
                               i:(i + twidth)]
            hist = ch.createHueSatHistogram(region, hist_size, is_weighted)
            bhattacharyya_coeff = np.sum(np.sqrt(hist_model * hist))
            similarity_map[j:(j + step), i:(i + step)] = bhattacharyya_coeff


    return similarity_map


def onMouse(event, x, y, flag, param):
    if event == cv2.EVENT_LBUTTONDOWN:
        param[0] = (x, y)
        param[1] = True

if __name__ == '__main__':
    cv2.namedWindow('input')

    #cap = cv2.VideoCapture('books.mp4')
    #cap = cv2.VideoCapture('vtest.avi')
    cap = cv2.VideoCapture(1)

    try:
        twidth = 64
        theight = 64
        tsize = (twidth, theight)

        hbin = 64
        sbin = 64
        hist_size = (hbin, sbin)
        hist_weighted = True
        #hist_weighted = False

        fig = plt.figure()
        ax = fig.gca(projection='3d')

        print("click to set the template")

        mouse_param = [ (-1, -1), False ]

        while True:
            retval, input_color = cap.read()
            input_hsv = cv2.cvtColor(input_color, cv2.COLOR_BGR2HSV)
            cv2.setMouseCallback('input', onMouse, mouse_param)

            tcenter = mouse_param[0]
            is_clicked = mouse_param[1]
            if is_clicked:
                model = cv2.getRectSubPix(input_hsv, (twidth, theight), tcenter)
                cv2.imshow("model", cv2.cvtColor(model, cv2.COLOR_HSV2BGR))
                hist_model = ch.createHueSatHistogram(model, hist_size,
                                                      hist_weighted)
                mouse_param[1] = False

            if tcenter != (-1, -1):
                step = 8
                similarity_map = colorSimilarity(input_hsv, hist_model, tsize, 
                                                 step, hist_weighted)
                cv2.imshow("similarity", similarity_map)

                cols = np.arange(0, input_hsv.shape[1] - twidth + 1, step)
                rows = np.arange(0, input_hsv.shape[0] - theight + 1, step)
                mesh_rows, mesh_cols = np.meshgrid(rows, cols)
                z = similarity_map[::step, ::step].T
                ax.plot_surface(mesh_rows, mesh_cols, z, cmap=cm.coolwarm)
                ax.set_xlabel('Y')
                ax.set_ylabel('X')
                plt.pause(0.001)
                ax.clear()

            cv2.imshow("input", input_color)
            key = cv2.waitKey(30)
            if key == ord('q'):
                break

    finally:
        cap.release()
        cv2.destroyAllWindows()
